import java.awt.*;
import java.awt.event.*;

public class CantFrame extends Frame implements ActionListener, ItemListener{

//Other parts of the interface
private Matrix matrix;	   //The grid
private final int TOPBORDER=100, LEFTBORDER=40;
private int NewRows=CANT20.net.Rows;
private int NewCols =CANT20.net.Cols; 

public static int DelayBetweenSteps;  //The algorithm sleeps between steps

//Things for the frame itself.
private Color _backColor;
private Label  TimeStepLabel;
private Label  OutputLabel;
public TextField UserParamField;
private int UserParamType=0; //1=Pattern Type, 2=Net Size, 3 Stimuli Active

private MenuBar bar;

private Menu netMenu;
private MenuItem newNet;
private MenuItem setNetRows;
private MenuItem setNetCols;
private MenuItem saveNet;
private MenuItem readNet;
private MenuItem setNetFile;
private MenuItem saveParameters;
private MenuItem readParameters;
private MenuItem setParameterFile;
private MenuItem printParameters;
private MenuItem exitCANT;

private Menu runMenu;
private MenuItem resetRun;
private MenuItem setDelay;
private MenuItem selectPattern;
private MenuItem PatternModifier; 
private MenuItem setStimuliActivated;
private MenuItem cyclesPerRun;
private CheckboxMenuItem changeEachTime;
private CheckboxMenuItem spontActPattern;
private CheckboxMenuItem associationTest;
private CheckboxMenuItem allowRunOn;
private CheckboxMenuItem neuronsFatigue;
private CheckboxMenuItem compensatoryLearningOn;
private CheckboxMenuItem learningOn;

private Menu setParametersMenu;   
private MenuItem Fatigue_Rate; //The following 8 items in Neuron.java
private MenuItem Fatigue_Recovery_Rate;
private MenuItem Decay;
private MenuItem Learning_Rate;
private MenuItem activationThreshold;
private MenuItem Compensatory_Divisor;
private MenuItem Axonal_Strength_Median;
private MenuItem Saturationbase;
private MenuItem likelihoodOfInhibitoryNeuron; // Decide inhibitory neurons
private MenuItem connectionStrength;
private MenuItem generateNetType;

private Menu measureMenu;
private MenuItem patternVariance;
private MenuItem printWeights;
private MenuItem printConnections;
private MenuItem measureCo1;
private MenuItem measureCo2;
private MenuItem printCorrelation;

//An extra window to print correlations etc. to.
private static Frame outputWindow;
private static TextArea outputTextArea = new TextArea();



public CantFrame() {
   super ("CANT Simulator");

   DelayBetweenSteps=50;

   //create the grid
   matrix = new Matrix(CANT20.net.Cols,CANT20.net.Rows,
                       LEFTBORDER,TOPBORDER);

   setSize (200+(Matrix.MatrixSize),
            200+(Matrix.MatrixSize));

   setBackground(_backColor=new Color(100,130,200));

   //Create buttons and other controls
   TimeStepLabel = new Label("TStep:            ");
   OutputLabel = new Label("Active Neurons:      ");
   UserParamField = new TextField(20);

   Button enterParamButton = new Button("Enter Parameter");
   enterParamButton.addActionListener(this);

   Button stepButton = new Button("Step");
   stepButton.addActionListener(this);

   Button startButton = new Button("Start");
   startButton.addActionListener(this);

   //sets up the page arrangement
   setLayout(new FlowLayout());

   //put the controls on the page
   add(TimeStepLabel);
   add(UserParamField);
   add(enterParamButton);
   add(startButton);
   add(stepButton);
   add(OutputLabel);
   
   //Create menus
   createMenus();
   setVisible(true);
   setLocation(200,0);
   
   //extra window for output from measurements.
   outputWindow = new Frame("CANT output");
   outputWindow.setSize(200,200);
   outputWindow.setLocation(600,0);
   outputWindow.setVisible(false);
   outputWindow.add(outputTextArea);
   outputWindow.addWindowListener( new WindowAdapter ()
      {
      public void windowClosing(WindowEvent w)
	     {
		 	outputWindow.setVisible(false);
		 }
      });
   
   addWindowListener( new CloseExit());
}

private void createMenus() {
   bar = new MenuBar();

   //Net Menu
   netMenu = new Menu ("Net");
   newNet = new MenuItem("New Network");
   netMenu.add(newNet);
   newNet.addActionListener(this);
   setNetCols = new MenuItem("Set New Network Cols");
   netMenu.add(setNetCols);
   setNetCols.addActionListener(this);
   setNetRows = new MenuItem("Set New Network Rows");
   netMenu.add(setNetRows);
   setNetRows.addActionListener(this);
   netMenu.addSeparator();
   saveNet = new MenuItem("Save Network");
   netMenu.add(saveNet);
   saveNet.addActionListener(this);
   readNet = new MenuItem("Read Network");
   netMenu.add(readNet);
   readNet.addActionListener(this);
   setNetFile = new MenuItem("Set Network File");
   netMenu.add(setNetFile);
   setNetFile.addActionListener(this);
   saveParameters = new MenuItem("Save Parameters");
   netMenu.add(saveParameters);
   saveParameters.addActionListener(this);
   readParameters = new MenuItem("Read Parameters");
   netMenu.add(readParameters);
   readParameters.addActionListener(this);
   setParameterFile = new MenuItem("Set Parameter File");
   netMenu.add(setParameterFile);
   setParameterFile.addActionListener(this);
   printParameters = new MenuItem("Print Parameters");
   netMenu.add(printParameters);
   printParameters.addActionListener(this);
   exitCANT = new MenuItem("Exit CANT");
   netMenu.add(exitCANT);
   exitCANT.addActionListener(this);
   bar.add(netMenu);

   //execute menu
   runMenu = new Menu ("Execution");
   resetRun = new MenuItem("Reset Run");
   runMenu.add(resetRun);
   resetRun.addActionListener(this);
   setDelay = new MenuItem("Set Run Delay");
   runMenu.add(setDelay);
   setDelay.addActionListener(this);
   selectPattern = new MenuItem("Select Pattern");
   runMenu.add(selectPattern);
   selectPattern.addActionListener(this);
   setStimuliActivated = new MenuItem("Set Stimuli Number");
   runMenu.add(setStimuliActivated);
   PatternModifier = new MenuItem("Pattern Modifier ");
   runMenu.add(PatternModifier);
   PatternModifier.addActionListener(this);
   setStimuliActivated.addActionListener(this);
   changeEachTime = new CheckboxMenuItem("Change Pattern Each Time");
   runMenu.add(changeEachTime);
   changeEachTime.addItemListener(this);
   changeEachTime.setState(CANT20.net.ChangeEachTime);
   cyclesPerRun = new MenuItem("Cycles Per Run");
   runMenu.add(cyclesPerRun);
   cyclesPerRun.addActionListener(this);
   runMenu.addSeparator();

   spontActPattern = new CheckboxMenuItem("Spontaneous Activation");
   runMenu.add(spontActPattern);
   spontActPattern.addItemListener(this);
   spontActPattern.setState(CANT20.net.SpontaneousActivationOn);
   associationTest = new CheckboxMenuItem("Association Test On");
   runMenu.add(associationTest);
   associationTest.addItemListener(this);
   allowRunOn = new CheckboxMenuItem("Allow Run On");
   runMenu.add(allowRunOn);
   allowRunOn.addItemListener(this);
   neuronsFatigue = new CheckboxMenuItem("Neurons Fatigue");
   runMenu.add(neuronsFatigue);
   neuronsFatigue.setState(CANT20.net.NeuronsFatigue);
   neuronsFatigue.addItemListener(this);
  
   runMenu.addSeparator();
   compensatoryLearningOn = new CheckboxMenuItem("Compensatory Learning On");
   runMenu.add(compensatoryLearningOn);
   compensatoryLearningOn.addItemListener(this);
   compensatoryLearningOn.setState(CANT20.net.CompensatoryLearningOn);
   learningOn = new CheckboxMenuItem("Learning On");
   runMenu.add(learningOn);
   learningOn.addItemListener(this);
   learningOn.setState(CANT20.net.LearningOn);
   bar.add(runMenu);


  //Set Parameters Menu
  setParametersMenu = new Menu ("Set Parameters");
  Fatigue_Rate = new MenuItem("Fatigue Rate");
  setParametersMenu.add(Fatigue_Rate);
  Fatigue_Rate.addActionListener(this);
  Fatigue_Recovery_Rate = new MenuItem("Fatigue Recovery Rate");
  setParametersMenu.add(Fatigue_Recovery_Rate);
  Fatigue_Recovery_Rate.addActionListener(this);
  Decay = new MenuItem("Decay Rate");
  setParametersMenu.add(Decay);
  Decay.addActionListener(this);
  activationThreshold = new MenuItem("Activation Threshold");
  setParametersMenu.add(activationThreshold);
  activationThreshold.addActionListener(this);
  Learning_Rate = new MenuItem("Learning Rate");
  setParametersMenu.add(Learning_Rate);
  Learning_Rate.addActionListener(this);
 
  setParametersMenu.addSeparator();
  Compensatory_Divisor = new MenuItem("Compensatory Divisor");
  setParametersMenu.add(Compensatory_Divisor);
  Compensatory_Divisor.addActionListener(this);
  Axonal_Strength_Median = new MenuItem("Synaptic Strength Median");
  setParametersMenu.add(Axonal_Strength_Median);
  Axonal_Strength_Median.addActionListener(this);
  Saturationbase = new MenuItem("Saturation Base");
  setParametersMenu.add(Saturationbase);
  Saturationbase.addActionListener(this);
  likelihoodOfInhibitoryNeuron = new MenuItem("Set Inhibitory Neuron");
  setParametersMenu.add(likelihoodOfInhibitoryNeuron);
  likelihoodOfInhibitoryNeuron.addActionListener(this);
  connectionStrength = new MenuItem("Base Connection Weights");
  setParametersMenu.add(connectionStrength);
  connectionStrength.addActionListener(this);
  generateNetType = new MenuItem("Generated Network Type");
  setParametersMenu.add(generateNetType);
  generateNetType.addActionListener(this);
  bar.add(setParametersMenu);

   //measurement menu
   measureMenu = new Menu ("Measure");
   patternVariance = new MenuItem("Print Synaptic Variance");
   measureMenu.add(patternVariance);
   patternVariance.addActionListener(this);
   printWeights = new MenuItem("Print Weights");
   measureMenu.add(printWeights);
   printWeights.addActionListener(this);
   printConnections = new MenuItem("Print Connections");
   measureMenu.add(printConnections);
   printConnections.addActionListener(this);
   measureCo1 = new MenuItem("Set Correlation Step1");
   measureMenu.add(measureCo1);
   measureCo1.addActionListener(this);
   measureCo2 = new MenuItem("Set Correlation Step2");
   measureMenu.add(measureCo2);
   measureCo2.addActionListener(this);   
   printCorrelation = new MenuItem("Print Correlation");
   measureMenu.add(printCorrelation);
   printCorrelation.addActionListener(this);
   bar.add(measureMenu);
   setMenuBar(bar);
}

public void paint(Graphics __g)
{
   setBackground(_backColor);
   matrix.paint(__g);
}

//Catch events from the frame (buttons menus)
public void actionPerformed (ActionEvent evt) {
    Object src=evt.getSource();

    if (evt.getSource() instanceof Button)
    {
      if (((Button)evt.getSource()).getLabel().equals("Start"))
         Start();

      if (((Button)evt.getSource()).getLabel().equals("Enter Parameter"))
         EnterUserParameter();

      if (((Button)evt.getSource()).getLabel().equals("Step")) {
         CANT20.isRunning=false;
         RunOneCycle(CANT20.inputPattern);
      }
    } // end if button

   else if (evt.getSource()== resetRun) {
      //set the run state to the initial state
	  CANT20.net.Clear();
	  CANT20.net.CANTStep=0;
	  //clear the matrix
	  for (int i =0;i <CANT20.net.Rows; i++)
	  	for (int j =0;j <CANT20.net.Cols; j++)
	  		matrix.setState(i,j,0);
			
	  TimeStepLabel.setText("TStep:  " + CANT20.net.CANTStep);	
	  OutputLabel.setText("Active Neurons: " );
			
	  repaint();
	  
   }
   else if (evt.getSource()== setDelay) {
      //Set state for the UserParam Box
      UserParamField.setText(Integer.toString(DelayBetweenSteps));
      UserParamType=23;
   }
   else if (evt.getSource()== selectPattern) {
      //Set state for the UserParam Box
      UserParamField.setText(
	     Integer.toString(CANT20.inputPattern.patternType));
      UserParamType=1;
   }
   else if (evt.getSource()== newNet ) {
       //Set up a new network
	   boolean temp = CANT20.READPARAMETERS;
	   CANT20.READPARAMETERS = false;
       CANT20.net = new Net(NewCols, NewRows);
       CANT20.READPARAMETERS = temp;
       if (CANT20.generateNetType <= 0) 
          CANT20.net.SetConnections(0,(NewCols*NewRows));
       else 	  {
          CANT20.net.SetConnections(0,(NewCols*NewRows),
		     CANT20.generateNetType);
       }	 

       //set up a new matrix
       matrix = new Matrix(NewCols,NewRows,LEFTBORDER,TOPBORDER);
       setSize (200+(Matrix.MatrixSize),200+(Matrix.MatrixSize));
   }
   else if (evt.getSource()== setNetCols ) {
      //Set state for the UserParam Box
      UserParamField.setText(
	     Integer.toString(CANT20.net.Cols));
      UserParamType=2;
   }
	  
   else if (evt.getSource() == readNet ) {
     if (CANT20.ApplicationNotApplet) 
        CANT20.net.Read();
     else CANT20.READNET = true;
   }
   else if (evt.getSource() == saveNet ) {
      if (CANT20.ApplicationNotApplet)
        CANT20.net.Write();
   }
   else if (evt.getSource() == setNetFile ) {
      //Set state for the UserParam Box
      UserParamField.setText(CANT20.net.NetFileName);
      UserParamType=19;
   }
   else if (evt.getSource() == printWeights ) {
      CANT20.net.PrintWeights();
   }
   else if (evt.getSource() == printConnections ) {
      CANT20.net.PrintConnections();
   }
   else if (evt.getSource() == setStimuliActivated ) {
      //Set state for the UserParam Box
      UserParamField.setText(Integer.toString(CANT20.net.NeuronsToStimulate));
      UserParamType=3;
   }

   else if (evt.getSource()== Fatigue_Rate ) {
      //Set state for the UserParam Box
      UserParamField.setText(Float.toString(Neuron.FATIGUE_RATE));
      UserParamType=4; }


   else if (evt.getSource()== Decay ) {
      //Set state for the UserParam Box
      UserParamField.setText(Float.toString(Neuron.DECAY));
      UserParamType= 5;   }


   else if (evt.getSource()== Fatigue_Recovery_Rate ) {
      //Set state for the UserParam Box
      UserParamField.setText(Float.toString(Neuron.FATIGUE_RECOVERY_RATE));
      UserParamType=6;   }


   else if (evt.getSource()== Axonal_Strength_Median ) {
      //Set state for the UserParam Box
      UserParamField.setText(Double.toString(Neuron.AXONAL_STRENGTH_MEDIAN));
      UserParamType=7; }

   else if (evt.getSource()== Learning_Rate ) {
      //Set state for the UserParam Box
      UserParamField.setText(Float.toString(Neuron.LEARNING_RATE));
      UserParamType=8;}

   else if (evt.getSource()== Compensatory_Divisor ) {
      //Set state for the UserParam Box
      UserParamField.setText(Integer.toString(Neuron.COMPENSATORY_DIVISOR));
      UserParamType=9;}

   else if (evt.getSource()== Saturationbase ) {
      UserParamField.setText(Float.toString(Neuron.SATURATIONBASE));
      UserParamType=10; }

   else if (evt.getSource()== likelihoodOfInhibitoryNeuron) {
      UserParamField.setText(
	     Integer.toString(Neuron.LIKELIHOOD_OF_INHIBITORY_NEURON));
      UserParamType=11;  }

   else if (evt.getSource()== cyclesPerRun) {
      UserParamField.setText(
	     Integer.toString(CANT20.inputPattern.CYCLESPERRUN));
      UserParamType=12; }

   else if (evt.getSource()== activationThreshold) {
      UserParamField.setText(Double.toString(Net.ACTIVATION_THRESHOLD));
      UserParamType=13; }

   else if (evt.getSource()== connectionStrength) {
      UserParamField.setText(Double.toString(Net.Conn_Str));
      UserParamType=14; }

   else if (evt.getSource()== setNetRows ) {
      //Set state for the UserParam Box
      UserParamField.setText(
        Integer.toString(CANT20.net.Rows));
      UserParamType=15;   }

   else if (evt.getSource()== generateNetType) {
        UserParamField.setText(Integer.toString(CANT20.generateNetType));
        UserParamType=16; }

   else if (evt.getSource()== measureCo1) {
      UserParamField.setText(Integer.toString(CANT20.net.MeasureCo1));
      UserParamType=17; }

   else if (evt.getSource()== measureCo2) {
      UserParamField.setText(Integer.toString(CANT20.net.MeasureCo2));
      UserParamType=18; }
		
   else if (evt.getSource() == setParameterFile )   {
      //Set state for the UserParam Box
      UserParamField.setText(CANT20.ParameterFileName);
      UserParamType=20;   }
	  
   else if (evt.getSource()== PatternModifier) {
      UserParamField.setText(Integer.toString(CANT20.inputPattern.patternModifier));
      UserParamType=25; }  
		  
   else if (evt.getSource() == saveParameters )  {
      if (CANT20.ApplicationNotApplet)
         Net.WriteParameters(); }
		 
   else if (evt.getSource() == readParameters )  {
      if (CANT20.ApplicationNotApplet) 
         Net.ReadParameters();
	  else CANT20.READPARAMETERS = true; }
	  
   else if (evt.getSource() == printParameters )
      System.out.println(
         "\n" + "Fatigue_Rate = "+Float.toString (Neuron.FATIGUE_RATE)
       + " Decay = " + Float.toString(Neuron.DECAY)
       + " Learning_Rate = " + Float.toString(Neuron.LEARNING_RATE)
       + " Fatigue_Recovery_Rate = " + Float.toString(Neuron.FATIGUE_RECOVERY_RATE) + "\n"
       + " Axonal_Strength_Median =" + Double.toString(Neuron.AXONAL_STRENGTH_MEDIAN)
       + " Saturationbase = " + Float.toString(Neuron.SATURATIONBASE)
       + " Set Inhibitory Neuron = " + Float.toString(Neuron.LIKELIHOOD_OF_INHIBITORY_NEURON) + "\n"
       + " Cycles Per Run = " + Integer.toString(CANT20.inputPattern.CYCLESPERRUN)
       + " Activation Threshold = " +  Double.toString(Net.ACTIVATION_THRESHOLD)
       + " Connection Stregnth = " +  Double.toString(Net.Conn_Str) + "\n"
       + " Generate Net Type = " +  Integer.toString(CANT20.generateNetType)
       + " NeuronsToStimulate = " + Integer.toString(Net.NeuronsToStimulate)
       + " LearningOn = " + (Net.LearningOn)  + "\n"
       + " SpontaneousActiva = " + (Net.SpontaneousActivationOn)
       + " AssociationTest = " + (Net.AssociationTest)
       + " AllowRunOn = " +  (Net.AllowRunOn) + "\n"
       + " ChangeEachTime = " +  (Net.ChangeEachTime)
       + " NeuronsFatigue = " + (Net.NeuronsFatigue) + "\n"
	   + " Compensatory Divisor =" + (Neuron.COMPENSATORY_DIVISOR) + "\n"
	     );

   else if (evt.getSource() == exitCANT )
   {
      System.out.print ( "exit CANT\n");
	  dispose();
	  System.exit(1);
   }
   
   else if (evt.getSource() == patternVariance ) {
      CANT20.net.printAxonalPatternVariance();   }
	  
   else if (evt.getSource() == printCorrelation ) {
   	String OutputString = "Step " + CANT20.net.MeasureCo1 + " and Step " +
	   CANT20.net.MeasureCo2 + " ";
	OutputString = OutputString + CANT20.net.Measure() + "\n";
   	WriteToOutputWindow(OutputString);   }   

   else
      System.out.print ( "uncaught user event\n");
}

public void itemStateChanged(ItemEvent evt) {
   if (evt.getSource() == changeEachTime ) {
      if (! this.changeEachTime.getState()) {
         CANT20.net.ChangeEachTime=false;
      }
      else {
         CANT20.net.ChangeEachTime=true;
      }
   }
   else if (evt.getSource() == associationTest ) {
      if (! this.associationTest.getState()) {
         CANT20.net.AssociationTest=false;
      }
      else {
         CANT20.net.AssociationTest=true;
      }
   }
   else if (evt.getSource() == allowRunOn ) {
      if (! this.allowRunOn.getState()) {
         CANT20.net.AllowRunOn=false;
      }
      else {
         CANT20.net.AllowRunOn=true;
      }
   }
   else if (evt.getSource() == neuronsFatigue) {
      if (! this.neuronsFatigue.getState()) {
         CANT20.net.NeuronsFatigue=false;
      }
      else {
         CANT20.net.NeuronsFatigue=true;
      }
   }
   else if (evt.getSource() == spontActPattern ) {
      if (! this.spontActPattern.getState()) {
         CANT20.net.SpontaneousActivationOn=false;
      }
      else {
         CANT20.net.SpontaneousActivationOn=true;
      }
   }
   else if (evt.getSource() == compensatoryLearningOn ) {
      if (! this.compensatoryLearningOn.getState()) {
         CANT20.net.CompensatoryLearningOn=false;
      }
      else {
         CANT20.net.CompensatoryLearningOn=true;
      }
   }   
   else if (evt.getSource() == learningOn ) {
      if (! this.learningOn.getState()) {
         CANT20.net.LearningOn=false;
      }
      else {
         CANT20.net.LearningOn=true;
      }
   }   
  
   else System.out.print ( "uncaught user item event\n");
}


private void Start() {
    CANT20.isRunning=true;
}

//Convert a positive integer string to a string.  Returns -1 on failure
private int StringToInteger(String Str){
   int IntValue;
   try{
      IntValue = (new Integer (Str)).intValue();
   }

   catch(NumberFormatException e) {
      return -1;
   }

   return IntValue;
}

//Depending on the Parameter Type (set internally) Change the Parameter.
//If it fails leave a message in the text box.
private void EnterUserParameter() {
   System.out.print ( "Parameter Type " + UserParamType + "\n");
   System.out.print ( "Parameter Value " + UserParamField.getText() + "\n");

   if (UserParamType == 1) {
      int NewPatternType = StringToInteger(UserParamField.getText());
      if ((NewPatternType > 0) && (NewPatternType < 16))
	 {
         CANT20.ChangePattern(NewPatternType);
         UserParamType = 0;
	 }
      else
         UserParamField.setText("Error Expecting Integer Pattern");
   }

   else if (UserParamType == 2) {
      int ColSize = StringToInteger(UserParamField.getText());
      if ((ColSize > 0) && (ColSize <= 50))
     {
         NewCols = ColSize;
         UserParamType = 0;
     }
   }

   else if (UserParamType == 3) {
      int NeuronsActivated = StringToInteger(UserParamField.getText());
      if (NeuronsActivated >= 0)
	 {
          CANT20.net.NeuronsToStimulate = NeuronsActivated;
          UserParamType = 0;
	 }
      else
         UserParamField.setText("Error Expecting Integer Below 200 Neurons Activated");
   }

      /*anita add the following program */
   else if (UserParamType ==4)  {

      String text4 = (UserParamField.getText());
      float Fatigue = Float.parseFloat(text4);
      if ((Fatigue>=0) && (Fatigue<=4))
          {Neuron.FATIGUE_RATE = Fatigue;}
      }

   else if (UserParamType ==5)  {
      float Decay = Float.parseFloat(UserParamField.getText());
      if (( Decay >= 1.0) && ( Decay <= 50.0))
          {Neuron.DECAY = Decay;}
      else
         UserParamField.setText("Error Expecting Float Between 1 and 50 Decay");
      }

   else if (UserParamType ==6)  {
      float Fatigue_Recovery_Rate = Float.parseFloat(UserParamField.getText());
      if ((Fatigue_Recovery_Rate >=0) && (Fatigue_Recovery_Rate<=4))
          {Neuron.FATIGUE_RECOVERY_RATE = Fatigue_Recovery_Rate;}
      }

   else if (UserParamType ==7)  {
      float Axonal_Strength_Median = Float.parseFloat(UserParamField.getText());
      if ((Axonal_Strength_Median>=0) && (Axonal_Strength_Median<=2))
          {
		  Neuron.AXONAL_STRENGTH_MEDIAN = Axonal_Strength_Median;
          }
      }
		  
   else if (UserParamType ==8)  {
      float Learning_Rate = Float.parseFloat(UserParamField.getText());
      if ((Learning_Rate>=0) && (Learning_Rate<=2))
          {Neuron.LEARNING_RATE = Learning_Rate;}
      }

   else if (UserParamType ==9)  {
      int Compensatory_Divisor = Integer.parseInt(UserParamField.getText());
      if ((Compensatory_Divisor > 0) && (Compensatory_Divisor <= 40))
          {Neuron.COMPENSATORY_DIVISOR = Compensatory_Divisor;}
      }

   else if (UserParamType == 10)  {
      float Saturationbase = Float.parseFloat(UserParamField.getText());
      if (( Saturationbase>=0) && ( Saturationbase<=60))
          {Neuron.SATURATIONBASE = Saturationbase;}
     }

     else if (UserParamType == 11)  {
      int inhibition =Integer.parseInt(UserParamField.getText());
      if (( inhibition>=0) && ( inhibition <=100))
          {Neuron.LIKELIHOOD_OF_INHIBITORY_NEURON = inhibition;}
     }
     else if (UserParamType == 12)  {
      int Cycles = Integer.parseInt(UserParamField.getText());
      if (( Cycles>= 1) && ( Cycles<= 1000))
          {CANT20.inputPattern.CYCLESPERRUN = Cycles;}
     }

    else if (UserParamType == 13)  {
      double Threshold = Float.parseFloat(UserParamField.getText());
      if (( Threshold >= 0.0) && ( Threshold <= 20.0))
          {Net.ACTIVATION_THRESHOLD = Threshold;}
     else
        UserParamField.setText("Error Expecting Float Below 20 Threshold");   }

    else if (UserParamType ==14)  {
      float conn_Str = Float.parseFloat(UserParamField.getText());
      if (( conn_Str >= 0.01) && ( conn_Str <= 0.6))
          {Net.Conn_Str = conn_Str;}
     }
    else if (UserParamType == 15) {
       int RowSize = StringToInteger(UserParamField.getText());
       if ((RowSize > 0) && (RowSize <= 50))
     {
          NewRows = RowSize;
          UserParamType = 0;
     }
    }

    else if (UserParamType ==16)  {
      int genNetType = Integer.parseInt(UserParamField.getText());
      if (( genNetType >= 1) && ( genNetType <= 6))
          {CANT20.generateNetType = genNetType;}
     }

    else if (UserParamType ==17)  {
      int Measureco1 = Integer.parseInt(UserParamField.getText());
      if (( Measureco1 >= 1) && ( Measureco1 <= 90000))
          {Net.MeasureCo1 = Measureco1;}
     }

     else if (UserParamType ==18)  {
      int Measureco2 = Integer.parseInt(UserParamField.getText());
      if (( Measureco2 >= 1) && ( Measureco2 <= 90000))
          {Net.MeasureCo2 = Measureco2;}
     }
     else if (UserParamType ==19)  {
        CANT20.net.NetFileName = UserParamField.getText();
     }
     else if (UserParamType ==20)  {
     	CANT20.ParameterFileName = UserParamField.getText();
     }

     else if (UserParamType ==21)  {
      if (UserParamField.getText() == "Y")
        Net.AssociationTest = true;
      else
        Net.AssociationTest = false;
     }

    else if (UserParamType ==22)  {
      if (UserParamField.getText() == "Y")
        Net.AllowRunOn = true;
      else
        Net.AllowRunOn = false;
     }
    else if (UserParamType ==23)  {
     DelayBetweenSteps = Integer.parseInt(UserParamField.getText());
    }
    else if (UserParamType ==24)  {
      if (UserParamField.getText() == "Y")
        Net.NeuronsFatigue = true;
      else
        Net.NeuronsFatigue = false;
     }


     else if (UserParamType ==25)  {    
      int patternModifier1 = Integer.parseInt(UserParamField.getText());
      if (( patternModifier1>= 0) && ( patternModifier1 <= 30000))
          {Inppat.patternModifier = patternModifier1 ;
		   System.out.print (Inppat.patternModifier); }
     }

   else
      System.out.print ( "Bad Parameter Type Value\n");

}

public void WriteToOutputWindow(String outputString)   
{
   outputWindow.setVisible(true);
   outputTextArea.appendText(outputString);
}   


public void RunOneCycle(Inppat inputPattern) {
	int ActiveNeurons;
	
    //Call the CANT net to run one step
    CANT20.net.RunOneStep(inputPattern);

    //Update the screen
    TimeStepLabel.setText("TStep:  " + CANT20.net.CANTStep);
    TimeStepLabel.repaint();
    
	ActiveNeurons = CANT20.net.GetActives();
    OutputLabel.setText("Active Neurons: " + ActiveNeurons);
    OutputLabel.repaint();

    int rows=matrix.maxY();
    int cols=matrix.maxX();

    int NeuronNum=0;

    for (int row = 0; row < rows; row++) {
        for (int col = 0; col < cols; col++) {
            NeuronNum = (row*cols) + col;
            if (CANT20.net.Neurons[NeuronNum].Fired)
                matrix.setState(col,row,1);
            else
                matrix.setState(col,row,0);
         }
    }

    try{
       Thread.sleep(DelayBetweenSteps);
    }

    catch(InterruptedException e) {
       System.out.print ( e.toString() + " \n");
    }

    repaint();
}



}//end of CantFrame class

