import java.io.*;
import java.util.*;

public class Neuron {
   final int AXONS_TO_ALLOC = 100;
//private:
   double CurrentActivation;
   boolean IsInhibitory;
   int TimeActive;
   float Fatigue;
   int ID;
   public Axon Axons[];
   public int CurrentAxons;
   boolean Fired = false;


//global constants
public static int LIKELIHOOD_OF_INHIBITORY_NEURON = 20;
public static float DECAY = (float) 1.5;  //CurrentAct = CurrentAct/DECAY
public static float FATIGUE_RATE = (float) 1.1;
public static float FATIGUE_RECOVERY_RATE = (float) 0.5;
public static float LEARNING_RATE = (float) 0.1;

//global variables
public static int GTotalNeurons = 0;
public static int LONGEST_TIME_NEURON_ACTIVE = 0;




public Neuron()
{
   double temp;
   
   CurrentActivation = 0;
   
   temp = Math.random()*100;
   if ((temp) < LIKELIHOOD_OF_INHIBITORY_NEURON)
   {
      IsInhibitory = true;
   }
   else
      IsInhibitory = false;
   TimeActive = 0;
   Fatigue = 0;
   ID = GTotalNeurons++;

   Axons = new Axon [AXONS_TO_ALLOC];
   CurrentAxons = 0;
}

   public double GetCurrentActivation() {return CurrentActivation;}
   public void SetActivation(double Activation) {CurrentActivation = Activation;};
   public boolean  GetInhibitory() {return (IsInhibitory);};
   public int GetID() {return ID;};

public boolean IsInhibitory() {return (IsInhibitory);}

public float GetTotalConnectionStrength ()
{
   float ConnectionStrength = (float)0.0;

   for (int oAxons=0; oAxons < CurrentAxons; oAxons++)
      ConnectionStrength +=  Axons[oAxons].GetWeight();

   return (ConnectionStrength);
}


//public void Read(DataInputStream InputFile) {
public void Read(LineNumberReader InputFile) {
   String inputLine,paramString;
   StringTokenizer tokenizedLine;
   int testID=-1;
   double Weight = 0.0;
   int toNeuronID = 0;
   int axonsToRead=0;
   Double tempDouble;
 
   //read the neuron ID
   try {
   	  inputLine = InputFile.readLine();
	  tokenizedLine = new StringTokenizer(inputLine);
	  paramString=tokenizedLine.nextToken();
      testID = Integer.parseInt(paramString);
   }
   catch (IOException e) {
      System.err.println("Bad Neuron Read" + e.toString());
      System.exit(1);
   }
		
   //assert(testID == ID);   // 8/12/02

   //read the number of axons
   try {
      inputLine = InputFile.readLine();
      tokenizedLine = new StringTokenizer(inputLine);
      paramString=tokenizedLine.nextToken();
      axonsToRead = Integer.parseInt(paramString);
   }
   catch (IOException e) {
       System.err.println("Bad Neuron Read" + e.toString());
       System.exit(1);
   }
      
   //read in the axons.
   for (int cAxons = 0; cAxons < axonsToRead; cAxons++) {
      try {
         inputLine = InputFile.readLine();
         tokenizedLine = new StringTokenizer(inputLine);
         paramString=tokenizedLine.nextToken();
         toNeuronID = Integer.parseInt(paramString);
         paramString=tokenizedLine.nextToken();
//         Weight = Double.parseDouble(paramString);
         tempDouble = new Double(paramString);
   	     Weight =  tempDouble.doubleValue();
      }
	  
      catch (IOException e) {
         System.err.println("Bad Axon Read" + e.toString());
         System.exit(1);
      }      	 
	  
	  //set the inhibitory neural value based on the first value
	  if (cAxons == 0)
	     if (Weight < 0) IsInhibitory = true;
         else IsInhibitory = false;

      AddConnection(CANT20.net.Neurons[toNeuronID],Weight);
   }
}
    

public  void ModifyFatigue()
{
    if (Fired)
	Fatigue += FATIGUE_RATE;
    else {
	Fatigue -= FATIGUE_RECOVERY_RATE;

        if (Fatigue < 0) Fatigue = 0;
    }
}


//undone put this generic stuff somewhere else.
//This is pretty generic and should be elsewhere
public void assert(boolean test)
{  int x = -1;
   if (! test) x = (1 / (1 +x));
}


//------Stuff for working between runs of the net-----------
 public void AddConnection(Neuron To, double Weight) {

   //assert (CurrentAxons < MaxAxons);

   //search for an existing connection.

   for (int cAxon = 0 ; cAxon < CurrentAxons; cAxon ++)
      if (Axons[cAxon].GetTo() == To)
         {
//         Axons[cAxon].SetWeight(Axons[cAxon].GetWeight() + Weight);
         Axons[cAxon].SetWeight(Weight);
         return;
         }
   //System.out.print(CurrentAxons+'\n');
   Axons[CurrentAxons] = new Axon();
   Axons[CurrentAxons].SetFrom(this);
   Axons[CurrentAxons].SetTo(To);
   Axons[CurrentAxons++].SetWeight(Weight);
}

public void Clear()
{
   CurrentActivation = 0;
   TimeActive = 0;
   Fatigue = 0;
}

//--------------------stuff for running the net-------------
public void SetFired()
{
   if ((CurrentActivation - Fatigue) >= Net.ACTIVATION_THRESHOLD)
      Fired = true;
   else
      Fired = false;
};

//Reset Activation and apply decay
public void ResetActivation()
{
   if (Fired) {
      CurrentActivation = 0;
      TimeActive++;
      //for printing the longest a neuron is active.
      if (TimeActive > LONGEST_TIME_NEURON_ACTIVE)
         LONGEST_TIME_NEURON_ACTIVE = TimeActive;
      assert (TimeActive > 0);
   }
   else {
      //if it is inactive cause it to decay
      CurrentActivation = CurrentActivation / DECAY;
   }
};

public void SpreadActivation()
{
   int AxonIndex;
   Neuron To;
   if (! Net.AssociationTest)
      //For each axon add the weight to its from node.
      for (AxonIndex = 0; AxonIndex < CurrentAxons; AxonIndex++)
      {
         To = Axons[AxonIndex].GetTo();
         To.SetActivation(To.CurrentActivation + Axons[AxonIndex].GetWeight());
      }
}

public boolean SpontaneouslyActivate()
{
   if (!Net.SpontaneousActivationOn) return false;

   //spontaneous activation
   if (Math.random() <= 0.01) {
       //if (Math.random() == 1.0) {
      //cout << ID << " spontaneous activation\n";
         return true;
   }
   return false;
}


////-----------------Stuff for the learning rules---------------------

public static int COMPENSATORY_DIVISOR = 10;
public static float SATURATIONBASE = 20;
public static double AXONAL_STRENGTH_MEDIAN = 0.5;

//If your strengthening (absolute value) increase a lot if the total weights
//are small, a little if they are large.
public double GetStrengthCompensatoryModifier(Neuron BaseNeuron)
{
   if (! Net.CompensatoryLearningOn) return 1.0;

   float TotalConnectionStrength = BaseNeuron.GetTotalConnectionStrength();
   double CompensatoryModifier,Result;

   if (TotalConnectionStrength < 0) TotalConnectionStrength *= -1.0;

   CompensatoryModifier = (SATURATIONBASE-TotalConnectionStrength)/
                           COMPENSATORY_DIVISOR;

   Result = Math.pow(5,CompensatoryModifier);

   return (Result);

}

//If you are weakening (closer to 0), move a lot if the total strengths are
//large and a little if they are small.
public double GetWeakCompensatoryModifier(Neuron FromNeuron)
{
   if (! Net.CompensatoryLearningOn) return 1.0;

   float TotalFromConnectionStrength = FromNeuron.GetTotalConnectionStrength();
   double CompensatoryFromModifier, Result;

   if (TotalFromConnectionStrength < 0) TotalFromConnectionStrength *= -1.0;

   CompensatoryFromModifier =
      Math.pow(5,((TotalFromConnectionStrength-SATURATIONBASE)/
         COMPENSATORY_DIVISOR));

   Result = CompensatoryFromModifier;

   /*
   //If to has large total activation, make the modifier even larger.
   if (CompensatoryToModifier > 1)
      Result *= CompensatoryToModifier;
   */

   //System.out.print ( "comw "+ TotalFromConnectionStrength + " " +
   //                     TotalToConnectionStrength + " " + Result + "\n");
   return (Result);
}

//How much to reduce an association based on Sterngth of the Connection
private double GetDecreaseBase(double CurrentStrength) {

   return (CurrentStrength*LEARNING_RATE);
}

//How much to increase an association based on strenth of the connection.
//TimesitDoesn'tFire*decrease_weight
//(1-Lij)*GetDecreaseBase
private double GetIncreaseBase(double CurrentStrength) {
  //Modified strength is 0-N strength normalised to 0-1
  double ModifiedStrength = CurrentStrength/(AXONAL_STRENGTH_MEDIAN*2);
  double EstimateNeuronNotFires = (1-ModifiedStrength); 

  return (EstimateNeuronNotFires*LEARNING_RATE*
     (AXONAL_STRENGTH_MEDIAN*2));
}


public void Learn(){

    Neuron ToNeuron;
    double ConnectionStrength;
    double Modification;
    double FromCompensatoryStrengthModifier,FromCompensatoryWeakModifier;
    double ToCompensatoryStrengthModifier = 1.0;
	double ToCompensatoryWeakModifier = 1.0;

    //Only learn when the from neuron is active
    if (!Fired) return;

    FromCompensatoryWeakModifier = GetWeakCompensatoryModifier(this);
    FromCompensatoryStrengthModifier = GetStrengthCompensatoryModifier(this);

    //Test each axon from the active neuron
    for (int oAxons=0; oAxons < CurrentAxons; oAxons++) {
	ConnectionStrength = Axons[oAxons].GetWeight();

        ToNeuron = Axons[oAxons].GetTo();

        //If both Neurons were active,
        if (ToNeuron.Fired) {
          if (this.GetInhibitory())
            {
            Modification =
               GetIncreaseBase(ConnectionStrength+(AXONAL_STRENGTH_MEDIAN*2));
            Modification *= FromCompensatoryWeakModifier * 
			   ToCompensatoryStrengthModifier;
            ConnectionStrength = ConnectionStrength+Modification;
            Axons[oAxons].SetWeight(ConnectionStrength);
            }
          else {
            Modification = GetIncreaseBase(ConnectionStrength);
            Modification *= FromCompensatoryStrengthModifier * 
			   ToCompensatoryStrengthModifier;			   
            ConnectionStrength = ConnectionStrength+Modification;
            Axons[oAxons].SetWeight(ConnectionStrength);
          }
        } //end of to neuron active

        //Weaken connections (anti-hebbian rule)
        else {
           if (this.GetInhibitory())
            {
            Modification =
               GetDecreaseBase(ConnectionStrength+(AXONAL_STRENGTH_MEDIAN*2));
            Modification *= FromCompensatoryStrengthModifier *
			   ToCompensatoryWeakModifier;
            ConnectionStrength = ConnectionStrength-Modification;
            Axons[oAxons].SetWeight(ConnectionStrength);
            }
          else {
            //Weaken activation
            Modification = GetDecreaseBase(ConnectionStrength);
            Modification *= FromCompensatoryWeakModifier *
			   ToCompensatoryWeakModifier;
            ConnectionStrength = ConnectionStrength-Modification;
            Axons[oAxons].SetWeight(ConnectionStrength);
          }
        }
    }
}


//:::::::::::::::::::::::::::::::::::::::::::
//Learning depends on a function, and it's inverse
//The above learning is implicitly based on a linear function
//This learning (below) is based on the learning function LF

double HEAT = -5.0;
double LNE = 2.718;
double LF (double Prob)
{
	double adjProb = (Prob*2) - 1.0;  //convert from 0-1 to -1-1
	double Base = Math.pow(LNE,HEAT*adjProb);
	double Result = 1/(1+Base);		
	return Result;
}

double LFInverse(double Wt) 
{
	double Base = Math.log((1/Wt)-1);
    double Result = (Base)/HEAT;
	Result = (Result + 1)/2;
	if (Result < 0.01) Result = 0.01;
	else if (Result > .99) Result = 0.99;
	return Result;
}

/*
double LF (double Prob) 
{
	return Prob;
}
double LFInverse (double Wt)
{
	return Wt;
}
*/
//A variant of learn that works on absolute value of synapses
//instead of deltas
public void Learn2() {
    Neuron ToNeuron;
    double oldProb, newProb, newWt;

    //Only learn when the from neuron is active
    if (!Fired) return;


    //Test each axon from the active neuron
    for (int oAxons=0; oAxons < CurrentAxons; oAxons++) {
	    ToNeuron = Axons[oAxons].GetTo();

        //If both Neurons were active,
        if (ToNeuron.Fired) {
        	if (!this.GetInhibitory())
		  	{
			oldProb = LFInverse(Axons[oAxons].Weight);
			newProb = oldProb + ((1-oldProb)*LEARNING_RATE);
			newWt = LF(newProb);
			Axons[oAxons].SetWeight(newWt);
//		  	System.out.print(oldProb + " " + newProb + " " + newWt +"+\n");
		  	}
			else
			{
			oldProb = LFInverse(1+Axons[oAxons].Weight);
		  	newProb = oldProb + ((1-oldProb)*LEARNING_RATE);
		  	newWt = LF(newProb);
		  	Axons[oAxons].SetWeight(newWt-1);
//		  	System.out.print(oldProb +" " + newWt + "+\n");
		  	}
        }
		else {	//!ToNeuron.Fired
			if (!this.GetInhibitory())
			{
			oldProb = LFInverse(Axons[oAxons].Weight);
			newProb = oldProb * (1-LEARNING_RATE);
			newWt = LF(newProb);
			Axons[oAxons].SetWeight(newWt);
			}
			else //make inhibition further from 0
			{
			oldProb = LFInverse(1+Axons[oAxons].Weight);
			newProb = oldProb * (1-LEARNING_RATE);
			newWt = LF(newProb);
			Axons[oAxons].SetWeight(newWt-1);
//			System.out.print(oldProb +" " + newWt + "-\n");
			}
		}
    }
}


}//end Neuron Class



